@@ -105,6 +105,7 @@ class XQuerySet
                 t->findAndReplace("\n"," ");
                 t->findAndReplace("\r"," ");
 		t->findAndReplace("@"," ");
+		t->findAndReplace("-","_");
 
 		h->trim();
 		t->trim();
@@ -373,6 +374,7 @@ class XNGram
 		d->findAndReplace("\n"," ");
 		d->findAndReplace("\r"," ");
 		d->findAndReplace("@"," ");
+		d->findAndReplace("-","_");
 
 		long i = d->indexOf(".");
 		if(i>=0)
@@ -468,25 +470,34 @@ class XNGram
 	}
 };
 
+static bool fts_backend_xapian_test_memory()
+{
+	void* p = malloc(XAPIAN_COMMIT_MEMORY * 1024);
+	if(p == NULL)
+	{
+		return false;
+	}
+	else
+	{
+		free(p);
+		return true;
+	}
+}
 
 static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend, Xapian::Database ** dbr)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_open_readonly");
+
 	if((backend->db == NULL) || (strlen(backend->db)<1))
 	{
 		if(verbose>0) i_warning("FTS Xapian: Open DB Read Only : no DB name");
 		return false;
 	}
 
-	struct stat sb;
-	if(!((stat(backend->db, &sb) == 0) && S_ISDIR(sb.st_mode)))
-	{
-		return false;
-	}
-        
 	try
 	{
 		if(verbose>1) i_info("FTS Xapian: Opening DB (RO) %s",backend->db);
-		*dbr = new Xapian::Database(backend->db,Xapian::DB_OPEN);
+		*dbr = new Xapian::Database(backend->db,Xapian::DB_CREATE_OR_OPEN);
 	}
 	catch(Xapian::Error e)
 	{
@@ -498,6 +509,8 @@ static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend,
 
 static bool fts_backend_xapian_check_access(struct xapian_fts_backend *backend)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_check_access");
+
 	if((backend->db == NULL) || (strlen(backend->db)<1))
 	{
 		if(verbose>0) i_warning("FTS Xapian: check_write : no DB name");
@@ -508,19 +521,22 @@ static bool fts_backend_xapian_check_access(struct xapian_fts_backend *backend)
 
 	try
 	{
-		if(verbose>1) i_info("FTS Xapian: Opening DB (RW) %s",backend->db);
+		if(verbose>0) i_info("FTS Xapian: Opening DB (RW) %s",backend->db);
 		backend->dbw = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK);
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: Can't open Xapian DB (%s) %s : %s",backend->boxname,backend->db,e.get_msg().c_str());
+		i_error("FTS Xapian: Can't open Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
 		return false;
 	}
+	if(verbose>0) i_info("FTS Xapian: Opening DB (RW) %s : Done",backend->db);
 	return true;
 }
 
 static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_oldbox");
+
 	if(backend->old_guid != NULL)
 	{
 		/* Performance calculator*/
@@ -544,6 +560,8 @@ static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 
 static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const char * reason, long commit_time)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_release (%s)",reason);
+
 	if(backend->dbw !=NULL)
         {
 		try
@@ -553,12 +571,13 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
                 }
                 catch(Xapian::Error e)
                 {
-                        i_error("FTS Xapian: (%s) %s",reason, e.get_msg().c_str());
+                        i_error("FTS Xapian: %s : %s - %s",reason,e.get_type(),e.get_error_string());
                 }
                 delete(backend->dbw);
-                backend->dbw=NULL;
-		backend->commit_updates=0;
+                backend->dbw = NULL;
+		backend->commit_updates = 0;
 		backend->commit_time = commit_time;
+		backend->memory = 0;
 	}
 
 	if(verbose>0)
@@ -572,6 +591,8 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 
 XResultSet * fts_backend_xapian_query(Xapian::Database * dbx, XQuerySet * query, long limit=0)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_query");
+
         XResultSet * set= new XResultSet();
 
 	try
@@ -602,13 +623,15 @@ XResultSet * fts_backend_xapian_query(Xapian::Database * dbx, XQuerySet * query,
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: %s",e.get_msg().c_str());
+		i_error("FTS Xapian: xapian_query %s - %s",e.get_type(),e.get_error_string());
 	}
 	return set;
 }
 
 static void fts_backend_xapian_do_expunge(struct xapian_fts_backend *backend, const char * reason, int limit)
 {
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_do_expunge");
+
 	if(!fts_backend_xapian_check_access(backend))
         {
 		if(verbose>0) i_info("FTS Xapian: Expunge on an empty box (%s)",reason);
@@ -632,7 +655,7 @@ static void fts_backend_xapian_do_expunge(struct xapian_fts_backend *backend, co
 	}
         catch(Xapian::Error e)
         {
-                i_error("FTS Xapian: Expunging '%s' : %s",backend->boxname,e.get_msg().c_str());
+                i_error("FTS Xapian: Expunging '%s' : %s - %s",backend->boxname,e.get_type(),e.get_error_string());
 		fts_backend_xapian_release(backend,"do_expunge",current_time);
 		return;
         }
@@ -660,7 +683,7 @@ static void fts_backend_xapian_do_expunge(struct xapian_fts_backend *backend, co
 			}
 			catch(Xapian::Error e)
 			{
-				i_error("FTS Xapian: Expunging UID=%d '%s' : %s",docid,backend->boxname,e.get_msg().c_str());
+				i_error("FTS Xapian: Expunging UID=%d '%s' : %s - %s",docid,backend->boxname,e.get_type(),e.get_error_string());
 			}
 		}
 		k++;
@@ -681,7 +704,7 @@ static void fts_backend_xapian_do_expunge(struct xapian_fts_backend *backend, co
 
 static int fts_backend_xapian_unset_box(struct xapian_fts_backend *backend)
 {
-	if(verbose>0) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
+	if(verbose>1) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
 
 	struct timeval tp;
         gettimeofday(&tp, NULL);
@@ -720,7 +743,7 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 	const char * mb;
 	fts_mailbox_get_guid(box, &mb );
 
-	if(verbose>0) i_info("FTX Xapian: Set box '%s' (%s)",box->name,mb);
+	if(verbose>1) i_info("FTX Xapian: Set box '%s' (%s)",box->name,mb);
 
 	if((mb == NULL) || (strlen(mb)<3))
 	{
@@ -742,11 +765,12 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
         gettimeofday(&tp, NULL);
         current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
 
-        backend->commit_updates=0;
+        backend->commit_updates = 0;
         backend->commit_time = current_time;
 	backend->guid = i_strdup(mb);
 	backend->boxname = i_strdup(box->name);
 	backend->db = i_strdup_printf("%s/db_%s",backend->path,mb);
+	backend->memory = 0;
 	
         /* Performance calculator*/
         backend->perf_dt = current_time;
@@ -823,8 +847,16 @@ static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *
         }
 }
 
-bool fts_backend_xapian_index_hdr(Xapian::WritableDatabase * dbx, uint uid, const char* field, icu::UnicodeString* data,long p, long f)
+bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid, const char* field, icu::UnicodeString* data)
 {
+	bool ok=true;
+
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_index_hdr");
+
+	Xapian::WritableDatabase * dbx = backend->dbw;
+	long p = backend->partial;
+	long f = backend->full;
+
 	if(data->length()<p) { return true; }
 
 	if(strlen(field)<1) { return true; }
@@ -838,162 +870,191 @@ bool fts_backend_xapian_index_hdr(Xapian::WritableDatabase * dbx, uint uid, cons
 
 	const char * h = hdrs_xapian[i];
 
+	XQuerySet * xq = new XQuerySet();
+        char *u = i_strdup_printf("%d",uid);
+        xq->add("uid",u);
+	i_free(u);
+
+        XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
+
+	Xapian::docid docid;
+	Xapian::Document * doc = NULL;
 	try
 	{
-		XQuerySet * xq = new XQuerySet();
-        	char *u = i_strdup_printf("%d",uid);
-        	xq->add("uid",u);
-		i_free(u);
-
-        	XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
-
-		Xapian::docid docid;
-		Xapian::Document doc;
 		if(result->size<1)
-        	{
-			doc.add_value(1,Xapian::sortable_serialise(uid));
+       		{
+			doc = new Xapian::Document();
+			doc->add_value(1,Xapian::sortable_serialise(uid));
 			u = i_strdup_printf("Q%d",uid);
-			doc.add_term(u);
-			docid=dbx->add_document(doc);
+			doc->add_term(u);
+			docid=dbx->add_document(*doc);
 			i_free(u);
         	}
 		else
 		{
 			docid=result->data[0];
-			doc = dbx->get_document(docid);
+			doc = new Xapian::Document(dbx->get_document(docid));
 		}
-		delete(result);
-		delete(xq);
-	
-		XNGram * ngram = new XNGram(p,f,h);
-	        ngram->add(data);
-	
-		if(verbose>0) 
-		{ 
-			i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024); 
-		}
-	
-		for(i=0;i<ngram->size;i++)
-		{
-			u = i_strdup_printf("%s%s",h,ngram->data[i]);
-			try
-			{
-				doc.add_term(u);
-			}
-			catch(Xapian::Error e)
-			{
-				i_error("FTS Xapian: %s",e.get_msg().c_str());
-			}
-			i_free(u);
-		}
-		delete(ngram);
-
-		dbx->replace_document(docid,doc);
-	    	return true;
 	}
 	catch(Xapian::Error e)
 	{
-		std::string s;
-		data->toUTF8String(s);
-		i_error("FTS Xapian: fts_backend_xapian_index_hdr (%s) -> %s",field,s.c_str());
-		i_error("FTS Xapian: %s",e.get_msg().c_str());
+		i_error("FTS Xapian: fts_backend_xapian_index_hdr : %s - %s",e.get_type(),e.get_error_string());
+		if(doc!=NULL) delete(doc);
+		ok=false;
 	}
-	return false;
+		
+	delete(result);
+	delete(xq);
+
+	if(!ok) return false;
+
+	XNGram * ngram = new XNGram(p,f,h);
+	ngram->add(data);
+	
+	if(verbose>0) 
+	{ 
+		i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024); 
+	}
+
+	backend->memory = backend->memory + ngram->memory;
+	
+	for(i=0;i<ngram->size;i++)
+	{
+		u = i_strdup_printf("%s%s",h,ngram->data[i]);
+		try
+		{
+			doc->add_term(u);
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: fts_backend_xapian_index_hdr : %s - %s",e.get_type(),e.get_error_string());
+			ok=false;
+		}
+		i_free(u);
+	}
+	delete(ngram);
+
+	if(ok) dbx->replace_document(docid,*doc);
+
+	delete(doc);
+
+	return ok;
 }
 
-bool fts_backend_xapian_index_text(Xapian::WritableDatabase * dbx,uint uid, const char * field, icu::UnicodeString * data,long p, long f)
+bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid, const char * field, icu::UnicodeString * data)
 {
+	bool ok = true;
+
+	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_index_text");
+
+	Xapian::WritableDatabase * dbx = backend->dbw;
+        long p = backend->partial;
+        long f = backend->full;
+
 	if(data->length()<p) { return true; }
 
+	XQuerySet * xq = new XQuerySet();
+
+	const char *u = t_strdup_printf("%d",uid);
+	xq->add("uid",u);
+
+	XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
+
+	Xapian::docid docid;
+	Xapian::Document * doc = NULL;
+
 	try
-        {
-		XQuerySet * xq = new XQuerySet();
-
-		const char *u = t_strdup_printf("%d",uid);
-		xq->add("uid",u);
-
-                XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
-  
-                Xapian::docid docid;
-                Xapian::Document doc;
-                if(result->size<1)
-                {
-			doc.add_value(1,Xapian::sortable_serialise(uid));
-                        u = t_strdup_printf("Q%d",uid);
-                        doc.add_term(u);
-                        docid=dbx->add_document(doc);
-                }
-                else
-                {
-			docid=result->data[0];
-                        doc = dbx->get_document(docid);
-                }
-		delete(result);
-		delete(xq);
-
-		Xapian::Document doc2;
-		Xapian::TermGenerator termgenerator;
-		Xapian::Stem stem("en");
-		termgenerator.set_stemmer(stem);
-		termgenerator.set_document(doc2);
-	
-		const char * h;
-                if(strcmp(field,"subject")==0) 
-		{
-			h="S";
+	{
+		if(result->size<1)
+        	{
+			doc = new Xapian::Document();
+			doc->add_value(1,Xapian::sortable_serialise(uid));
+			u = t_strdup_printf("Q%d",uid);
+			doc->add_term(u);
+                	docid=dbx->add_document(*doc);
 		}
 		else
 		{
-			h="XBDY";
+			docid=result->data[0];
+			doc = new Xapian::Document(dbx->get_document(docid));
 		}
-		std::string s;
-		data->toUTF8String(s);
-		termgenerator.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
-		termgenerator.index_text(s, 1, h);
+	}
+	catch(Xapian::Error e)
+	{
+		i_error("FTS Xapian: fts_backend_xapian_index_text : %s - %s",e.get_type(),e.get_error_string());
+		if(doc!=NULL) delete(doc);
+		ok=false;
+        }
+
+	delete(result);
+	delete(xq);
+
+	if(!ok) return false;
+
+	Xapian::Document * doc2 = new Xapian::Document();
+	Xapian::TermGenerator termgenerator;
+	Xapian::Stem stem("en");
+	termgenerator.set_stemmer(stem);
+	termgenerator.set_document(*doc2);
+
+	const char * h;
+	if(strcmp(field,"subject")==0) 
+	{
+		h="S";
+	}
+	else
+	{
+		h="XBDY";
+	}
+	std::string s;
+	data->toUTF8String(s);
+	termgenerator.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
+	termgenerator.index_text(s, 1, h);
 	
-		long l = strlen(h);
-		long n = doc2.termlist_count();
-		Xapian::TermIterator ti = doc2.termlist_begin();
-		XNGram * ngram = new XNGram(p,f,h);
-		const char * c;
-		while(n>0)
+	long l = strlen(h);
+	long n = doc2->termlist_count();
+	Xapian::TermIterator ti = doc2->termlist_begin();
+
+	XNGram * ngram = new XNGram(p,f,h);
+	const char * c;
+	while(n>0)
+	{
+		s = *ti;
+		c=s.c_str();	
+		if(strncmp(c,h,l)==0)
 		{
-			s = *ti;
-			c=s.c_str();	
-			if(strncmp(c,h,l)==0)
-			{
-				ngram->add(c+l);
-			}
-			ti++;
-			n--;
+			ngram->add(c+l);
 		}
-		if(verbose>0) i_info("FTS Xapian: NGRAM(%s,%s) -> %ld items, max length=%ld, (total %ld KB)",field,h,ngram->size,ngram->maxlength,ngram->memory/1024);
+		ti++;
+		n--;
+	}
 
-		char *t = (char*)i_malloc(sizeof(char)*(ngram->maxlength+6));
-		for(n=0;n<ngram->size;n++)
+	if(verbose>0) i_info("FTS Xapian: NGRAM(%s,%s) -> %ld items, max length=%ld, (total %ld KB)",field,h,ngram->size,ngram->maxlength,ngram->memory/1024);
+	backend->memory = backend->memory + ngram->memory;
+	delete(doc2);
+
+	char *t = (char*)i_malloc(sizeof(char)*(ngram->maxlength+6));
+	for(n=0;n<ngram->size;n++)
+	{
+		snprintf(t,ngram->maxlength+6,"%s%s",h,ngram->data[n]);
+                try
                 {
-                        snprintf(t,ngram->maxlength+6,"%s%s",h,ngram->data[n]);
-                        try
-                        {
-                                doc.add_term(t);
-                        }
-                        catch(Xapian::Error e)
-                        {
-                                i_error("FTS Xapian: %s",e.get_msg().c_str());
-                        }
-                }
-                i_free(t);
-                delete(ngram);
+			doc->add_term(t);
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: xapian_index_text : %s - %s",e.get_type(),e.get_error_string());
+			ok=false;
+		}
+	}
+	i_free(t);
+	delete(ngram);
 
-                dbx->replace_document(docid,doc);
-          }
-          catch(Xapian::Error e)
-          {
-		i_error("FTS Xapian: fts_backend_xapian_index_text");
-		i_error("FTS Xapian: %s",e.get_msg().c_str());
-		return false;
-          }
-          return true;
+	if(ok) dbx->replace_document(docid,*doc);
+
+	delete(doc);
+
+	return ok;
 }
 
 static int fts_backend_xapian_empty_db_remove(const char *fpath, const struct stat *sb, int typeflag)
