@@ -11,10 +11,11 @@ extern "C" {
 
 #define XAPIAN_FILE_PREFIX "xapian-indexes"
 #define XAPIAN_TERM_SIZELIMIT 245
-#define XAPIAN_COMMIT_LIMIT 1000
+#define XAPIAN_COMMIT_ENTRIES 1000000
 #define XAPIAN_COMMIT_TIMEOUT 300
+#define XAPIAN_COMMIT_MEMORY 10240
 #define XAPIAN_WILDCARD "wldcrd"
-#define XAPIAN_EXPUNGE_SIZE 2
+#define XAPIAN_EXPUNGE_SIZE 3
 #define XAPIAN_EXPUNGE_HEADER 9
 
 #define HDRS_NB 11
@@ -43,11 +44,12 @@ struct xapian_fts_backend
 	long commit_updates;
 	long commit_time;
 
+	long memory;
+
 	long perf_pt;
 	long perf_nb;
 	long perf_uid;
 	long perf_dt;
-
 };
 
 struct xapian_fts_backend_update_context
@@ -72,6 +74,8 @@ static struct fts_backend *fts_backend_xapian_alloc(void)
 
 static int fts_backend_xapian_init(struct fts_backend *_backend, const char **error_r)
 {
+	if(verbose>0) i_info("fts_backend_xapian_init");
+
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 	const char *const *tmp, *env;
@@ -166,12 +170,10 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 
 static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_deinit");
-
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 
-	if(verbose>1) i_info("Deinit %s",backend->path);
+	if(verbose>0) i_info("FTS Xapian: Deinit %s",backend->path);
 
 	if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
 
@@ -190,7 +192,7 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 
 static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct mailbox *box, uint32_t *last_uid_r)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_get_last_uid");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_get_last_uid");
 
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
@@ -203,23 +205,30 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 		return -1;
 	}
 
-	if(!fts_backend_xapian_check_access(backend))
-	{
-		i_error("FTS Xapian: get_last_uid: can not open DB %s",backend->db);
-		return -1;
-	}
+	Xapian::Database * dbr;
+
+        if(!fts_backend_xapian_open_readonly(backend, &dbr))
+        {
+		if(verbose>0) i_info("FTS Xapian: GetLastUID: Can not open db RO (%s)",backend->db);
+                return 0;
+        }
 
     	try
 	{
-		*last_uid_r = Xapian::sortable_unserialise(backend->dbw->get_value_upper_bound(1));
+		*last_uid_r = Xapian::sortable_unserialise(dbr->get_value_upper_bound(1));
 	}
 	catch(Xapian::Error e)
 	{
 		i_error("FTS Xapian: fts_backend_xapian_get_last_uid for '%s' (%s)",backend->boxname,backend->guid);
 		i_error("FTS Xapian: %s",e.get_msg().c_str());
+		dbr->close();
+		delete(dbr);
 		return -1;
 	}
 
+	dbr->close();
+	delete(dbr);
+
 	if(verbose>0) i_info("FTS Xapian: Get last UID of %s (%s) = %d",backend->boxname,backend->guid,*last_uid_r);
 
         return 0;
@@ -228,7 +237,7 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 
 static struct fts_backend_update_context * fts_backend_xapian_update_init(struct fts_backend *_backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_update_context");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_update_context");
 
 	struct xapian_fts_backend_update_context *ctx;
 
@@ -239,14 +248,18 @@ static struct fts_backend_update_context * fts_backend_xapian_update_init(struct
 
 static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_ctx)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_update_deinit");
-
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)ctx->ctx.backend;
 
-	if(verbose>1) i_info("Update deinit %s",backend->path);
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_deinit (%s)",backend->path);
+
+	struct timeval tp;
+        gettimeofday(&tp, NULL);
+        long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+
+        fts_backend_xapian_release(backend,"update_deinit", current_time);
 
 	i_free(ctx);
 
@@ -255,7 +268,7 @@ static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_
 
 static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_context *_ctx, struct mailbox *box)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_update_set_mailbox");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_set_mailbox");
 
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
@@ -267,7 +280,7 @@ static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_cont
 
 static void fts_backend_xapian_update_expunge(struct fts_backend_update_context *_ctx, uint32_t uid)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_update_expunge");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_expunge");
 
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
@@ -457,7 +470,7 @@ static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_
 
 static int fts_backend_xapian_refresh(struct fts_backend * _backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_refresh");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_refresh");
 
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *) _backend;
@@ -498,11 +511,11 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
     	if(ctx->tbi_isfield)
     	{
-        	ok=fts_backend_xapian_index_hdr(backend->dbw,ctx->tbi_uid,ctx->tbi_field, &d2, backend->partial,backend->full);
+        	ok=fts_backend_xapian_index_hdr(backend,ctx->tbi_uid,ctx->tbi_field, &d2);
     	}
     	else
     	{
-		ok=fts_backend_xapian_index_text(backend->dbw,ctx->tbi_uid,ctx->tbi_field, &d2, backend->partial,backend->full);
+		ok=fts_backend_xapian_index_text(backend,ctx->tbi_uid,ctx->tbi_field, &d2);
     	}
 
 	backend->commit_updates++;
@@ -511,9 +524,9 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 	gettimeofday(&tp, NULL);
 	long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
 
-	if( (backend->commit_updates>XAPIAN_COMMIT_LIMIT) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) )
+	if( (backend->commit_updates>XAPIAN_COMMIT_ENTRIES) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) || (!fts_backend_xapian_test_memory()))
 	{
-		if(verbose>1) i_info("FTS Xapian: Refreshing after %ld ms and %ld updates...", current_time - backend->commit_time, backend->commit_updates);
+		if(verbose>0) i_info("FTS Xapian: Refreshing after %ld ms (vs %ld) and %ld updates (vs %ld) and %ld KB ...", current_time - backend->commit_time, XAPIAN_COMMIT_TIMEOUT*1000, backend->commit_updates, XAPIAN_COMMIT_ENTRIES, backend->memory/1024);
 		fts_backend_xapian_release(backend,"refreshing", current_time);
 	}
     	
@@ -523,7 +536,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
 static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_optimize");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_optimize");
 
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *) _backend;
@@ -540,7 +553,7 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 
 static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_rescan");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_rescan");
 
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
@@ -557,7 +570,7 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 
 static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbox *box, struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_result *result)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_lookup");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup");
 
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
@@ -577,7 +590,6 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 
 	if(!fts_backend_xapian_open_readonly(backend, &dbr))
         {
-                i_error("FTS Xapian: Lookup: Can not open db RO");
 		i_array_init(&(result->definite_uids),0);
                 return 0;
         }
@@ -634,7 +646,7 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 
 static int fts_backend_xapian_lookup_multi (struct fts_backend *_backend, struct mailbox *const boxes[], struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_multi_result *result)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_lookup_multi");
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup_multi");
 
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
